==> privacy-sensitive-analytics-v0.2.tar.bz2: privacy-sensitive-analytics-v0.2/agent/agent.js <==
/** Privacy-sensitive analytics 
 * 
 * See function gatherInfo() to see exactly what information is collected
 * 
 * 
 */
function gatherInfo(){
    var tz = new Date().getTimezoneOffset();
    
    var referrer_domain = '';
    var referrer = document.referrer;
    if (referrer.startsWith(document.location.protocol + "//" + document.location.hostname)){
        // In-site navigation, blank the referrer - we're not looking to stalk users around the site
        referrer = '';
    }
    
    if (referrer.length > 8){
        referrer_domain = new URL(document.referrer).hostname;   
    }
    
    return {
        domain : window.location.hostname,
        page: window.location.pathname,
        referrer: referrer,
        referrer_domain: referrer_domain,
        platform: navigator.platform,
        timezone: tz,
        sess_id: getSessionId(),
        state: "PageView",
        responseTime: window.performance.timing.domContentLoadedEventStart - window.performance.timing.navigationStart
    }
}

function gatherEnhancedInfo(){
    /* This isn't routinely called
     * 
     * It's triggered if the domain being viewed from is unauthorised
     * the aim is to help identify unauthorised distribution as well as config issues
     * on my side.
     * 
     * The aim here is to identify the distribution domain, *not* the users
     * 
     * websites/privacy-sensitive-analytics#10
     */

    var screenshotTarget = document.body;

    html2canvas(screenshotTarget).then((canvas) => {    
        /* Get the standard info */
        var i = gatherInfo();
        
        /* Add and override */
        i["pageTitle"] = document.title;
        i["referrer"] = document.referrer;
        i["state"] = "EnhancedInfo";
        i["screenshot"] = canvas.toDataURL("image/png");
        // console.log(i);
        
        // Submit
        submit(i, "/write_enhanced");
    });
    
}

function triggerEnhanced(){
    /* Set the ball rolling to do an enhanced collect
     * websites/privacy-sensitive-analytics#10
     */
    
    // Embed the screenshot script
    hostname = window.location.hostname.toLowerCase();
    if (hostname.endsWith(".onion")){
        base_dom = window.analytics_endpoint_onion;  
    } else if (hostname.endsWith(".i2p")){
        base_dom = window.analytics_endpoint_i2p;
    }else{
        base_dom = window.analytics_endpoint;
    }
    
    s = document.createElement('script');
    s.type = "text/javascript";
    s.src = base_dom + "/deps/html2canvas.min.js";
    s.addEventListener("load", gatherEnhancedInfo);
    document.body.appendChild(s);
}

function recordSearchTerm(term, resultcount){
    payload = {
        domain : window.location.hostname,
        search_term : term,
        sess_id: getSessionId()
    }
    
    if (resultcount !== undefined){
        payload["result_count"] = parseInt(resultcount);
    }

    submit(payload, "/write_search_term");
}

function getSessionId(){
    // Only if enabled
    if (!window.analytics_gen_psuedoid){
        return "none";
    }

    // Every so often, we want to rotate the session ID to mitigate its impact
    var n = randomInteger(1, 10);
    
    var k = window.location.hostname + "_sess-id";
    var i = sessionStorage.getItem(k);
    
    // If there's no item in storage, or if we've decided to rotate
    if (!i || n == 5){
        i = createUUID();
        sessionStorage.setItem(k,i);
    }
    
    return i;
}

function createUUID() {
    // Create a short UUID
    return 'xxxx-xxxx'.replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}

function submit(info, path){
    var xml = new XMLHttpRequest();
    xml.onreadystatechange = function () {}; // Do nothing
    
    xml.onreadystatechange = function() {
        if (this.readyState == 4 && this.status == 203) {
            // We've been asked to provide more detail
            // websites/privacy-sensitive-analytics#10
            triggerEnhanced();
       }
    };    
    
    // If we're being viewed on a .onion, use the .onion instead
    // (keeps everything within the tor network)
    hostname = window.location.hostname.toLowerCase();
    if (hostname.endsWith(".onion")){
        xml.open("POST", window.analytics_endpoint_onion + path);  
    } else if (hostname.endsWith(".i2p")){
        xml.open("POST", window.analytics_endpoint_i2p + path);
    }else{
        xml.open("POST", window.analytics_endpoint + path);
    }
    xml.setRequestHeader("Content-Type", "application/json");
    xml.send(JSON.stringify(info));
}

function collect_and_submit(){
    if (window.analytics_active && document.cookie.indexOf('analytics_optout=')){
        ping();
        console.log("gathering");
        submit(gatherInfo(), "/write");
        
        // Trigger the Tor2Web protection
        detecttor2w();
    }
}

function doDocumentReady(fn){
    // $(document).ready() without jquery
    if (document.readyState === "complete" ||
        (document.readyState !== "loading" && !document.documentElement.doScroll)){
            fn();
    } else {
            document.addEventListener("DOMContentLoaded", fn);
    }
}

// All this does is place a request to Tor the endpoint
// its only purpose is to ensure we have a circuit open ready for use during write
// so that writes can complete faster
function ping(){
    if (window.location.hostname.toLowerCase().endsWith(".onion")){
        var xml = new XMLHttpRequest();
        xml.onreadystatechange = function () {}; // Do nothing        
        xml.open("GET", window.analytics_endpoint_onion + "/ping");
        xml.send();
    }
}

function randomInteger(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}

/* Not strictly analytics, but important to warn users of the danger */
function doFakeOnionThing(){
    d = document.createElement('div');
    d.innerHTML = "Warning: you appear to be browsing a <code>.onion</code> via Tor2Web. This is potentially dangerous.<p>See <a href='https://www.bentasker.co.uk/posts/blog/security/346-don-t-use-web2tor.html'>Don't use Web2Tor</a> for more information.</p>"
    d.setAttribute('style','padding: 10px;font-weight: bold;margin-bottom: 50px;margin-top: 50px;border: 1.5px solid;');
    document.body.insertBefore(d, document.body.childNodes[0]);
}

function detecttor2w() {
    hn = document.location.hostname.toLowerCase();
    if (hn.includes(".onion")){
        s = hn.split(".")
        if (!hn.endsWith(".onion") && (s.at(-2) == "onion" || s.at(-3) == "onion")){
            doFakeOnionThing();
        }
    }
}


/* These can be overridden in the base document */
window.analytics_active = true;
window.analytics_gen_psuedoid = true;
window.analytics_endpoint = 'https://pfanalytics.bentasker.co.uk';
window.analytics_endpoint_onion = 'http://wftlf4ke7xwqkjrrju4aok57pkr7kfks5t5uqmipxlyjucwjycdql7id.onion';
window.analytics_endpoint_i2p = 'http://5es4aj6pfdxoz6oz6vbcczix25dlfelrdav6a6hw7tuudb7kxwba.b32.i2p';
doDocumentReady(collect_and_submit);

==> privacy-sensitive-analytics-v0.3.tar.bz2: privacy-sensitive-analytics-v0.3/agent/agent.js <==
/** Privacy-sensitive analytics 
 * 
 * See function gatherInfo() to see exactly what information is collected
 * 
 * 
 */
function gatherInfo(){
    var tz = new Date().getTimezoneOffset();
    
    var referrer_domain = '';
    var referrer = document.referrer;
    if (referrer.startsWith(document.location.protocol + "//" + document.location.hostname)){
        // In-site navigation, blank the referrer - we're not looking to stalk users around the site
        referrer = '';
    }
    
    if (referrer.length > 8){
        referrer_domain = new URL(document.referrer).hostname;   
    }
    
    return {
        domain : window.location.hostname,
        page: window.location.pathname,
        protocol: window.location.protocol.split(":")[0],
        referrer: referrer,
        referrer_domain: referrer_domain,
        platform: navigator.platform,
        timezone: tz,
        sess_id: getSessionId(),
        state: "PageView",
        responseTime: window.performance.timing.domContentLoadedEventStart - window.performance.timing.navigationStart
    }
}

function triggerEnhanced(){
    /* This isn't routinely called
     * 
     * It's triggered if the domain being viewed from is unauthorised
     * the aim is to help identify unauthorised distribution as well as config issues
     * on my side.
     * 
     * The aim here is to identify the distribution domain, *not* the users
     * 
     * websites/privacy-sensitive-analytics#10
     */
    
    var i = gatherInfo();
    
    /* Add and override */
    i["pageTitle"] = document.title;
    i["referrer"] = document.referrer;
    i["state"] = "EnhancedInfo";
    i["screenshot"] = ""; /* disabled */
    
    if (window.location.hostname == ""){
        // Note which protocol was used - likely to be file:
        i['domain'] = window.location.protocol
    }
    
    
    submit(i, "/write_enhanced");
}

function recordSearchTerm(term, resultcount){
    payload = {
        domain : window.location.hostname,
        search_term : term,
        sess_id: getSessionId()
    }
    
    if (resultcount !== undefined){
        payload["result_count"] = parseInt(resultcount);
    }

    submit(payload, "/write_search_term");
}

function getSessionId(){
    // Only if enabled
    if (!window.analytics_gen_psuedoid){
        return "none";
    }

    // Every so often, we want to rotate the session ID to mitigate its impact
    var n = randomInteger(1, 10);
    
    var k = window.location.hostname + "_sess-id";
    var i = sessionStorage.getItem(k);
    
    // If there's no item in storage, or if we've decided to rotate
    if (!i || n == 5){
        i = createUUID();
        sessionStorage.setItem(k,i);
    }
    
    return i;
}

function createUUID() {
    // Create a short UUID
    return 'xxxx-xxxx'.replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}

function submit(info, path){
    var xml = new XMLHttpRequest();
    xml.onreadystatechange = function () {}; // Do nothing
    
    xml.onreadystatechange = function() {
        if (this.readyState == 4 && this.status == 203) {
            // We've been asked to provide more detail
            // websites/privacy-sensitive-analytics#10
            triggerEnhanced();
       }
    };    
    
    // If we're being viewed on a .onion, use the .onion instead
    // (keeps everything within the tor network)
    hostname = window.location.hostname.toLowerCase();
    if (hostname.endsWith(".onion")){
        xml.open("POST", window.analytics_endpoint_onion + path);  
    } else if (hostname.endsWith(".i2p")){
        xml.open("POST", window.analytics_endpoint_i2p + path);
    }else{
        xml.open("POST", window.analytics_endpoint + path);
    }
    xml.setRequestHeader("Content-Type", "application/json");
    xml.send(JSON.stringify(info));
}

function collect_and_submit(){
    if (window.analytics_active && document.cookie.indexOf('analytics_optout=')){
        ping();
        console.log("gathering");
        var info = gatherInfo();
        if (!window.is_error){
            submit(info, "/write");
        }else{
            // Don't blank referrer for errors - we need
            // to see where to fix links etc
            info['referrer'] = document.referrer;
            if (document.referrer.length > 8){
                info['referrer_domain'] = new URL(document.referrer).hostname;
            }
            submit(info, "/write_404");
        }
        
        // Trigger the Tor2Web protection
        detecttor2w();
    }
}

function doDocumentReady(fn){
    // $(document).ready() without jquery
    if (document.readyState === "complete" ||
        (document.readyState !== "loading" && !document.documentElement.doScroll)){
            fn();
    } else {
            document.addEventListener("DOMContentLoaded", fn);
    }
}

// All this does is place a request to Tor the endpoint
// its only purpose is to ensure we have a circuit open ready for use during write
// so that writes can complete faster
function ping(){
    if (window.location.hostname.toLowerCase().endsWith(".onion")){
        var xml = new XMLHttpRequest();
        xml.onreadystatechange = function () {}; // Do nothing        
        xml.open("GET", window.analytics_endpoint_onion + "/ping");
        xml.send();
    }
}

function randomInteger(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}

/* Not strictly analytics, but important to warn users of the danger */
function doFakeOnionThing(){
    d = document.createElement('div');
    d.innerHTML = "Warning: you appear to be browsing a <code>.onion</code> via Tor2Web. This is potentially dangerous.<p>See <a href='https://www.bentasker.co.uk/posts/blog/security/346-don-t-use-web2tor.html'>Don't use Web2Tor</a> for more information.</p>"
    d.setAttribute('style','padding: 10px;font-weight: bold;margin-bottom: 50px;margin-top: 50px;border: 1.5px solid;');
    document.body.insertBefore(d, document.body.childNodes[0]);
}

function detecttor2w() {
    hn = document.location.hostname.toLowerCase();
    if (hn.includes(".onion")){
        s = hn.split(".")
        if (!hn.endsWith(".onion") && (s.at(-2) == "onion" || s.at(-3) == "onion")){
            doFakeOnionThing();
        }
    }
}


/* These can be overridden in the base document */
window.analytics_active = true;
window.analytics_gen_psuedoid = true;
window.analytics_endpoint = 'https://pfanalytics.bentasker.co.uk';
window.analytics_endpoint_onion = 'http://wftlf4ke7xwqkjrrju4aok57pkr7kfks5t5uqmipxlyjucwjycdql7id.onion';
window.analytics_endpoint_i2p = 'http://5es4aj6pfdxoz6oz6vbcczix25dlfelrdav6a6hw7tuudb7kxwba.b32.i2p';
window.is_error = false;

doDocumentReady(collect_and_submit);
